package net.sf.alchim.codeplus.spoonprocessor;

import java.util.ArrayList;

import spoon.processing.AbstractProcessor;
import spoon.processing.Property;
import spoon.reflect.declaration.CtAnnotation;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.ModifierKind;
import spoon.template.Substitution;

import net.sf.alchim.codeplus.annotation.GenStupidGetSet;

/**
 * This processor inserts the getters and the setters typical implementations
 * for all the fields annotated with the
 * {@link spoon.examples.fieldaccess.annotation.Access} annotation.
 */
public class StupidGetSetGenerator<_FieldType_> extends AbstractProcessor<CtField<_FieldType_>> {

    public static ArrayList<CtClass<?>> targetClasses_ = new ArrayList<CtClass<?>>();

    @Property
    public boolean generateOnlyOnAnnotation = true;

    /**
     * TODO: check if getter and setter not exist
     */
    public void process(CtField<_FieldType_> field) {
        try {
            CtClass<?> parent = field.getParent(CtClass.class);
            if (!targetClasses_.contains(parent)) {
                CtAnnotation<?> annotation = ProcessorHelper.findAnnotation(parent, GenStupidGetSet.class);
                if (generateOnlyOnAnnotation && (annotation == null)) {
                    return;
                }
                targetClasses_.add(parent);
                ProcessorHelper.removeAnnotation(parent, GenStupidGetSet.class);
            }

            if (field.getModifiers().contains(ModifierKind.STATIC)) {
                return;
            }
            if (field.getModifiers().contains(ModifierKind.FINAL)) {
                return;
            }
            if (!field.getModifiers().contains(ModifierKind.PUBLIC)) {
                return;
            }
            // fields.add(field.getReference());
            Substitution.insertAll(field.getParent(CtClass.class), new StupidGetSetTemplate<_FieldType_>(field.getType(), field.getSimpleName()));
        } catch (Exception exc) {
            throw new ProcessingException(this, field, exc);
        }
    }

}
